
/*--------------------------------------------------------------------------------------
                                Include section
----------------------------------------------------------------------------------------*/
#define STRICT
#include <windows.h>
#include <stdio.h>
#include <stdlib.h> 
#include <string.h>

#define G_WINDOWS
#include "c:/gemplus/api4/mpcos/samples/gemplus.h"             /* For Gemplus general declarations             */
#include "c:/gemplus/api4/mpcos/include/wscump2.h"             /* Smart Card Utilities for MPCOS cards         */


/*--------------------------------------------------------------------------------------
                                Constant section
----------------------------------------------------------------------------------------*/
/* Reader parameters                                                                    */
#define IFD_TYPE  GCR400        /* GCR400    type                                       */
#define IFD_MODE  G_SERIAL      /* serial communication                                 */
#define IFD_PORT  G_COM1        /* Reader Connected on COM 1                            */
#define IFD_BAUD  9600          /* Speed                                                */
#define IFD_IT    0xFF          /* Interrupt                                            */

                                 
#define SOFT_COMPUTE -1         /* This value , used in some MPCOS functions,           */
                                /* indicates that the calculation is made by            */
                                /* the terminal and not in the security module          */ 
                                
/*--------------------------------------------------------------------------------------
                             Function definition section
----------------------------------------------------------------------------------------*/
 
 
/****************************************************************************************
* INT16 OpenChannel(INT16 IFDType)
*
* Description : This function opens the communication with the card reader.
*
* Remarks     : Nothing.
*
* In          : IFDType: Type of card reader.
*
* Out         : Nothing.
*
* Response    : The open channel identifier.

*****************************************************************************************/
INT16 OpenChannel(INT16 IFDType){
   G4_CHANNEL_PARAM
      channel;
   INT16
      ChannelID;

   /* prepares channel parameters                                                       */
   channel.IFDType              = IFDType; 
   channel.IFDMode              = IFD_MODE;
   channel.Comm.Serial.Port     = IFD_PORT; 
   channel.Comm.Serial.BaudRate = IFD_BAUD;
   channel.Comm.Serial.ITNumber = IFD_IT;
      

   /* opens communication with the reader                                               */
   printf("* Open Channel: ");
   ChannelID = G4_OpenChannel(&channel);
   if (ChannelID < G_OK)
   {
      printf(" Failure.    G4_OpenChannel returns %d.\n",ChannelID);
   }
   else
   {
      printf("OK.\n");
   }

   return ChannelID;
}
/****************************************************************************************
* void  CloseChannel(INT16 ChannelID)
*
* Description : This function closes the communication with a card reader
*
* Remarks     : Nothing.
*
* In          : ChannelID: The Channel identifier.
*
* Out         : Nothing.

*****************************************************************************************/
void CloseChannel(INT16 ChannelID)
{
   INT16
      response;

   printf("* Close Channel: ");
   
   response = G4_CloseChannel(ChannelID);
   
   if (response < G_OK){
      printf(" Failure  ");
      printf("G4_CloseChannel returns %d.\n",response);
   }
   else{
      printf("OK.\n");
   }
}

/****************************************************************************************
* void  OpenSession  (INT16 ChannelID)
*
* Description : This function opens a session with a MPCOS card, Initializes the 
*               library and sets the value of the IO buffer size according to the
*               type of the card.
*
* Remarks     : Nothing.
*
* In          : ChannelID: The channel identifier.
*
* Out         : Nothing.
*
*****************************************************************************************/
void OpenSession(INT16 ChannelID)
{
   INT16
      response;
   G4_SESSION_PARAM 
      session; 
   WORD8 
      DFDescriptor[18],
      szIssuingData[12],
      ComponentReference, 
      CodeReference, 
      Version;             
   WORD16 
      nCardType; 


   /* Open session --------------------------------------------------------------------*/
  
   session.ICCType = ISOCARD ;
  
   printf("* Open Session: ");
   
   response = G4_OpenSession(ChannelID,
                             &session);
   if (response < G_OK){
      printf(" Failure.  ");
      printf("G4_OpenSession returns %d.\n",response);
   }
   else{
      printf("OK.\n");   
   }

   /* Init library ---------------------------------------------------------------------*/
   
   printf("* Init Library: ");
   response = G_MPC_InitLib (ChannelID,
                             DFDescriptor);
   if (response < G_OK){
      printf(" Failure.  ");
      printf("G_MPC_InitLib returns %d.\n",response);
   }
   else{
      printf("OK.\n");   
   }                 

  /* Get Info --------------------------------------------------------------------------*/
  
   printf("* Get Info: ");
   response = G_MPC_GetInfo (ChannelID,
                             0xA4,          /* Code to recover the chip type infos      */
                             12,            /* length expected                          */
                             szIssuingData);
   if (response < G_OK){
      printf(" Failure.  ");
      printf("G_MPC_GetInfo returns %d.\n",response);
   }
   else{
      printf("OK.\n");   
   }                 
   
   ComponentReference = szIssuingData[0];
   CodeReference      = szIssuingData[1];
   Version            = szIssuingData[2];
   
   
  /* Set IO size -----------------------------------------------------------------------*/    
  
      nCardType = UNKNOWN_CHIP;
   
   /* These are the common component references and versions of MPCOS cards             */
   
      if (CodeReference == 0xA0){
      if ((ComponentReference == 0x25) && (Version == 0x52))   nCardType = MPCOS24K;
      if ((ComponentReference == 0x30) && (Version == 0x52))   nCardType = MPCOS64K;
      if ((ComponentReference == 0x30) && (Version == 0x55))   nCardType = MPCOS64K;
      if ((ComponentReference == 0x31) && (Version == 0x5A))   nCardType = MPCOS64K;
      if ((ComponentReference == 0x32) && (Version == 0x54))   nCardType = MPCOS8K;
      if ((ComponentReference == 0x3B) && (Version == 0x56))   nCardType = MPCOS32K;
      if ((ComponentReference == 0x3C) && (Version == 0x59))   nCardType = MPCOS32K;
   }
                               
   switch(nCardType){  
   
    case  MPCOS8K : G_MPC_SetIOSize(16); /* Buffer size is 16 for MPCOS8K               */
                    break;
    case  MPCOS24K: G_MPC_SetIOSize(32); /* Buffer size is 32 for MPCOS24K              */
                    break;
    case  MPCOS32K: G_MPC_SetIOSize(96); /* Buffer size is 96 for MPCOS32K              */
                    break;
    case  MPCOS64K: G_MPC_SetIOSize(96); /* Buffer size is 96 for MPCOS64K              */
                    break;                                                           
   /* If the card is not in the list the default value of the IO buffer size is 16      */
    default       : G_MPC_SetIOSize(16); 
                    break;                                                           
   }            
                    

}

/****************************************************************************************
* void  CloseSession(INT16 ChannelID)
*
* Description : This function closes the session with a card
*
* Remarks     : Nothing.
*
* In          : ChannelID: The Channel identifier.
*
* Out         : Nothing.

*****************************************************************************************/
void CloseSession(INT16 ChannelID)
{
   INT16 
      response;
   
   printf("* Close Session: ");
   
   response = G4_CloseSession(ChannelID);
   
   if (response < G_OK){
      printf(" Failure.  ");
      printf("G4_CloseSession returns %d.\n",ChannelID);
   }
   else{
      printf("OK.\n");
   }
 
}           

/****************************************************************************************
* void  SelectDF(INT16 ChannelID,WORD16 DFIdentifier)
*
* Description : This function selects a DF using its identifier.
*
* Remarks     : Nothing.
*
* In          : ChannelID: The Channel identifier
*               DFIdentifier: The DF identifier.
*
* Out         : Nothing.

*****************************************************************************************/
void SelectDF(INT16 ChannelID, WORD16 DFIdentifier)    
{
   WORD8 
      Descriptor[36];
   INT16 
      Mode,
      response;                        
   
   Mode = 1;              /* The mode is 1 to select a DF file                          */
    
   printf("* Select DF: ");
    
   response = G_MPC_Select(ChannelID, 
                           Mode, 
                           DFIdentifier, 
                           Descriptor);                          
  
            
   /* Descriptor contains  "85" +                                                       */
   /*                       "10"(size of the DF descriptor in hexa) +                   */
   /*                        the 16 bytes of the DF decriptor +                         */
   /*                        "84" +                                                     */
   /*                        the length of the DF name +                                */
   /*                        the DF name (up to 16 bytes).                              */
                                                                                      

   if (response < G_OK){
         printf(" Failure.  ");
         printf("G_MPC_Select returns %d.\n",response);
   }
   else{
         printf("OK.\n");
   }
     
}

/****************************************************************************************
* INT16  SelectPurseKey(INT16 ChannelID,WORD16 KeyEFIdentifier,
*                     WORD16 PurseEFIdentifier, WORD8 *Random, WORD8 *CardResult)
*
* Description : This function selects a purse and a key and establish a new
*               session key.
*
* Remarks     : Nothing.
*
* In          : ChannelID:         The Channel identifier        
*               KeyEFIdentifier:   The key file identifier
*               PurseEFIdentifier: The purse identifier
*
* Out         : Random:     The random string.
*               CardResult: The result returned by the card.
*
* Response    : The Card Transaction Counter
*
*****************************************************************************************/
 INT32 SelectPurseKey(INT16 ChannelID,WORD16 KeyEFIdentifier,
                     WORD16 PurseEFIdentifier, WORD8 *Random, WORD8 *CardResult)
{
   INT16 
      response,
      KeyNumber;
   INT32 
      CTC;      /* Card transaction Counter                                             */
   int i;


   for (i=0;i<8;i++)
   Random[i] = (unsigned char) rand();
     
   KeyNumber = 0;       /* key number within the key file                               */
   
   printf("* Select Purse Key: ");
    
    
   response = G_MPC_SelPurseKey(ChannelID, 
                                KeyEFIdentifier,
                                KeyNumber,
                                PurseEFIdentifier,
                                Random,
                                CardResult,
                                &CTC
                               );  

   if (response < G_OK){
         printf(" Failure.  ");
         printf("G_MPC_SelPurseKey returns %d.\n",response);
   }
   else{
         printf("OK.\n");                               
         
   }
   return (CTC);
}   
/****************************************************************************************
* void  Authentication(WORD16 PurseEFIdentifier,INT32 CTC,
*                      WORD8* PaymentKey, WORD8 *Random, WORD8 *CardResult)
*
* Description : This function allows the terminal to authenticate the card.
*
* Remarks     : Nothing.
*
* In          : PurseEFIdentifier: The purse identifier        
*               CTC:               The Card Transaction Counter
*               PaymentKey:        The payment key
*               Random:            The random string build during the select 
                                   purse key step.
*               CardResult:        The result returned by the card during the 
                                   select purse key step.
*
* Out         : Nothing.
*
*****************************************************************************************/
void Authentication(WORD16 PurseEFIdentifier,INT32 CTC,
                      WORD8* PaymentKey, WORD8 *Random, WORD8 *CardResult)
{
   INT16
      response; 
    
   printf("* Authentication: ");

   response = G_MPCSM_SetPaymentSessionKey(SOFT_COMPUTE, 
                                           PurseEFIdentifier,
                                           CTC,
                                           Random,
                                           CardResult,
                                           PaymentKey
                                          );

   if (response < G_OK){
         printf(" Failure.  ");
         printf("G_MPCSM_SetPaymentSessionKey returns %d.\n",response);
   }
   else{
         printf("OK.\n");
   }
   
}

/****************************************************************************************
* void  ReadBalance(INT16 ChannelID,WORD16 PurseEFIdentifier,
*                   WORD16 TCCFIX,WORD16 TCCVAR)
*
* Description : This function reads the balance of the previous transactions.
*
* Remarks     : Nothing.
*
* In          : ChannelID:         The Channel identifier        
*               PurseEFIdentifier: The purse identifier
*               TCCFIX:            The terminal identifier
*               TCCVAR:            The terminal counter
*
* Out         : Nothing.
*
*****************************************************************************************/
void ReadBalance(INT16 ChannelID,WORD16 PurseEFIdentifier,WORD16 TCCFIX,WORD16 TCCVAR)
{
   INT16 
      response; 
   INT32 
      Balance;
   WORD8 
      CardBalanceCertificate[2];
    
    
   printf("* Read Balance: ");
    
   response = G_MPC_ReadBalance(ChannelID, 
                                PurseEFIdentifier,
                                TCCFIX,
                                TCCVAR,
                                &Balance,
                                CardBalanceCertificate
                               );

   if (response < G_OK){
         printf(" Failure.  ");
         printf("G_MPC_ReadBalance returns %d.\n",response);
   }
   else{
         printf("OK.");                               
         printf("    Balance = %d\n",Balance);
         
   }
}   
/****************************************************************************************
* void  HostCreditCryptogram(INT32 lCreditValue, WORD8 G_FAR *CreditKey, 
*                     WORD8 G_FAR *szHostCrypt)
*
* Description : This function is used to send a credit from the host to a terminal.
*               It computes the credit cryptogram.
*
* Remarks     : Nothing.
*
* In          : lCreditValue: The credit value        
*               CreditKey:    The credit key
*
* Out         : szHostCrypt: The host credit cryptogram string.
*
*****************************************************************************************/
void HostCreditCryptogram(INT32 lCreditValue, WORD8 G_FAR *CreditKey, 
                      WORD8 G_FAR *szHostCrypt)
{
   INT16 
      response; 
   
                          
    
   printf("* Host credit cryptogram: ");
    
    
   response = G_MPCSM_HostCreditCryptogram(SOFT_COMPUTE, 
                                           lCreditValue,
                                           CreditKey,
                                           szHostCrypt
                                          );

   if (response < G_OK){
         printf(" Failure.  ");
         printf("G_MPCSM_HostCreditCryptogram returns %d.\n",response);
   }
   else{
         printf("OK.\n");                               
   }
   
}   
/****************************************************************************************
* void  TermCreditCryptogram(WORD8 G_FAR *szHostCrypt, 
                      WORD8 G_FAR *szTermCrypt)
*
* Description : This function computes the credit cryptogram to be sent to 
*               the card.
*
* Remarks     : Nothing.
*
* In          : szHostCrypt: The host credit cryptogram string determined during 
*                            the host credit cryptogram step    
*
* Out         : szTermCrypt: The terminal credit cryptogram string.
*
*****************************************************************************************/
void TermCreditCryptogram(WORD8 G_FAR *szHostCrypt, 
                      WORD8 G_FAR *szTermCrypt)
{
   INT16
      response; 
   
   printf("* Term credit cryptogram: ");
    
    
   response = G_MPCSM_TermCreditCryptogram(SOFT_COMPUTE, 
                                           szHostCrypt,
                                           szTermCrypt
                                          );

   if (response < G_OK){
         printf(" Failure.  ");
         printf("G_MPCSM_TermCreditCryptogram returns %d.\n",response);
   }
   else{
         printf("OK.\n");                               
   }
}  
 
/****************************************************************************************
* void  Credit(INT16 ChannelID,INT32 lCreditValue, WORD8 G_FAR *szTermCrypt)
*
* Description : This function credit a purse.
*
* Remarks     : Nothing.
*
* In          : ChannelID:    The channel identifier.
*               lCreditValue: The value of the credit
*               szTermCrypt:  The terminal credit cryptogram string determinated 
*                             during the terminal credit cryptogram step 
*
* Out         : nothing.
*
*****************************************************************************************/
void Credit(INT16 ChannelID,INT32 lCreditValue, WORD8 G_FAR *szTermCrypt)
{
   INT16
      response,
      KeyNumber;
    
   KeyNumber = 2;      /* Credit key number within the key file                         */
   
   printf("* Credit : ");
    
    
   response = G_MPC_CreditPurse(ChannelID, 
                                KeyNumber,
                                lCreditValue,
                                szTermCrypt
                               );

   if (response < G_OK){
         printf(" Failure.  ");
         printf("G_MPC_CreditPurse returns %d.\n",response);
   }
   else{
         printf("OK.");
         printf("    Credit value = %d\n",lCreditValue);                               
         
      }
 
}   

/****************************************************************************************
* void  main()
*
* Description : Main program.
*               Executes the different steps of a purse credit.
*
* Remarks     : Nothing.
*
* In          : nothing
* Out         : nothing.
*
*****************************************************************************************/
void main()
{ 
   INT16 
      ChannelID;               /* Channel Identifier                                    */
   WORD16 
      DFIdentifier,            /* DF Identifier                                         */
      KeyEFIdentifier,         /* Key file identifier                                   */
      PurseEFIdentifier,       /* Purse file identifier                                 */
      TCCFIX,                  /* Terminal identifier                                   */
      TCCVAR;                  /* Terminal counter                                      */
   WORD8
      CreditKey[16],           /* Credit key                                            */
      PaymentKey[16],          /* Payment key                                           */
      Random[8],               /* Random string initialized in the  SelectPurseKey step */
      CardResult[4],           /* Card result returned in the SelectPurseKey step       */
      szHostCrypt[8],          /* Host credit cryptogram string                         */
      szTermCrypt[8];          /* Terminal credit cryptogram string                     */
   WORD32 
      CTC;                     /* Card Transaction Counter                              */
   INT32 
      lCreditValue;            /* Value of the credit                                   */
   int i;

/* The following initializations are common to the 3 programs MPCPurse, MPCCred, MPCDeb */

   for (i=0;i<8;i++)  PaymentKey[i] = 0x12;   /* Initialization of the payment key      */
   for (i=8;i<16;i++) PaymentKey[i] = 0x56;   /* Initialization of the payment key      */
   for (i=0;i<8;i++)  CreditKey[i] = 0x34;    /* Initialization of the credit key       */
   for (i=8;i<16;i++) CreditKey[i] = 0x78;    /* Initialization of the credit key       */
   
   TCCFIX = 0x1234;                  /* Initialization of the terminal identifier       */
   TCCVAR = 0xABCD ;                 /* Initialization of the terminal counter          */

 
/* Sample information text                                                              */

   printf("--------------------------------------------------------------------------\n");
   printf("MPCOS-3DES Library                                                        \n");
   printf("Reader: GCR 400 Serial/9600 on COM 1                                      \n");
   printf("--------------------------------------------------------------------------\n");
   printf("This sample shows the different steps in the credit of a purse.           \n");
   printf("                                                                          \n");
   printf("The program asks you to enter a DF identifier. From this value it creates \n");
   printf("the other keys and the default values as follows.                         \n");
   printf("  Purse EF Identifier               =   DF Identifier + 1                 \n");
   printf("  Key EF Identifier                 =   DF Identifier + 2                 \n");
   printf("  Credit value                      =   100                               \n");
   printf("                                                                          \n");
   printf("For each step of the purse credit, the sample displays the name of the    \n");
   printf("function used and the status returned.                                    \n");
   printf("                                                                          \n");
   printf("--------------------------------------------------------------------------\n");
   printf("                                                                          \n");
   printf("press <ENTER> to run the sample.                                          \n");
   printf("                                                                          \n");

   getchar();
   
/* The value of the DF is the only parameter asked to the user                          */

   printf("Enter the DF identifier XXXX (hexa):"); 
   scanf("%04X",&DFIdentifier);
   
/* The bulding of the Purse file identifier and the key file identifier from the value  */
/* of the DF identifier is common to the 3 programs MPCPurse, MPCCred, MPCDeb           */

   PurseEFIdentifier        =   DFIdentifier + 1;
   KeyEFIdentifier          =   DFIdentifier + 2; 
         
   printf(" DFIdentifier      %04X\n",DFIdentifier);
   printf(" PurseEFIdentifier %04X\n",PurseEFIdentifier);
   printf(" KeyEFIdentifier   %04X\n",KeyEFIdentifier);
         
/* The value of the credit is set to 100 for this sample                                */
   lCreditValue = 100;
        


   ChannelID = OpenChannel(IFD_TYPE);              /* Open channel                      */
   if (ChannelID >= G_OK) {
      
         OpenSession(ChannelID);                   /* Open session                      */
         
          
         SelectDF(ChannelID,DFIdentifier);         /* Select DF                         */
         
         CTC = SelectPurseKey(ChannelID,           /* Select purse key                  */
                              KeyEFIdentifier,
                              PurseEFIdentifier,
                              Random,
                              CardResult);
         Authentication(PurseEFIdentifier,         /* Authentication                  */
                          CTC,
                          PaymentKey,
                          Random,
                          CardResult);
         ReadBalance(ChannelID,                    /* Read balance                      */
                     PurseEFIdentifier,
                     TCCFIX,
                     TCCVAR);
         HostCreditCryptogram(lCreditValue,        /* Host credit cryptogram            */
                              CreditKey,
                              szHostCrypt);
         TermCreditCryptogram(szHostCrypt,         /* Terminal credit cryptogram        */
                              szTermCrypt);
         Credit(ChannelID,                         /* Credit                            */
                lCreditValue,                      
                szTermCrypt);
         ReadBalance(ChannelID,                    /* Read balance                      */
                     PurseEFIdentifier,
                     TCCFIX,
                     TCCVAR);
         CloseSession(ChannelID);                  /* Close session                     */
         CloseChannel(ChannelID);                  /* Close channel                     */
   }

}